@@ -12,9 +12,11 @@ import com.fasterxml.jackson.core.format.InputAccessor;
 import com.fasterxml.jackson.core.format.MatchStrength;
 import com.fasterxml.jackson.core.io.*;
 import com.fasterxml.jackson.core.json.*;
+import com.fasterxml.jackson.core.json.async.NonBlockingJsonParser;
 import com.fasterxml.jackson.core.sym.ByteQuadsCanonicalizer;
 import com.fasterxml.jackson.core.sym.CharsToNameCanonicalizer;
 import com.fasterxml.jackson.core.util.BufferRecycler;
+import com.fasterxml.jackson.core.util.BufferRecyclers;
 import com.fasterxml.jackson.core.util.DefaultPrettyPrinter;
 
 /**
@@ -111,18 +113,20 @@ public class JsonFactory
          * {@link ThreadLocal} (see
          * <a href="https://github.com/FasterXML/jackson-core/issues/189">Issue #189</a>
          * for a possible case)
+         *<p>
+         * This setting is enabled by default.
          *
          * @since 2.6
          */
         USE_THREAD_LOCAL_FOR_BUFFER_RECYCLING(true)
-        
+
         ;
 
         /**
          * Whether feature is enabled or disabled by default.
          */
         private final boolean _defaultState;
-        
+
         /**
          * Method that calculates bit set (flags) of all features that
          * are enabled by default.
@@ -179,14 +183,6 @@ public class JsonFactory
     /**********************************************************
      */
 
-    /**
-     * This <code>ThreadLocal</code> contains a {@link java.lang.ref.SoftReference}
-     * to a {@link BufferRecycler} used to provide a low-cost
-     * buffer recycling between reader and writer instances.
-     */
-    final protected static ThreadLocal<SoftReference<BufferRecycler>> _recyclerRef
-        = new ThreadLocal<SoftReference<BufferRecycler>>();
-
     /**
      * Each factory comes equipped with a shared root symbol table.
      * It should not be linked back to the original blueprint, to
@@ -290,7 +286,7 @@ public class JsonFactory
      */
     protected JsonFactory(JsonFactory src, ObjectCodec codec)
     {
-        _objectCodec = null;
+        _objectCodec = codec;
         _factoryFeatures = src._factoryFeatures;
         _parserFeatures = src._parserFeatures;
         _generatorFeatures = src._generatorFeatures;
@@ -404,6 +400,20 @@ public class JsonFactory
      */
     public boolean canUseCharArrays() { return true; }
 
+    /**
+     * Introspection method that can be used to check whether this
+     * factory can create non-blocking parsers: parsers that do not
+     * use blocking I/O abstractions but instead use a
+     * {@link com.fasterxml.jackson.core.async.NonBlockingInputFeeder}.
+     *
+     * @since 2.9
+     */
+    public boolean canParseAsync() {
+        // 31-May-2017, tatu: Jackson 2.9 does support async parsing for JSON,
+        //   but not all other formats, so need to do this:
+        return _isJSONFactory();
+    }
+
     /**
      * Method for accessing kind of {@link FormatFeature} that a parser
      * {@link JsonParser} produced by this factory would accept, if any;
@@ -726,7 +736,7 @@ public class JsonFactory
 
     /*
     /**********************************************************
-    /* Parser factories (new ones, as per [Issue-25])
+    /* Parser factories, traditional (blocking) I/O sources
     /**********************************************************
      */
 
@@ -914,6 +924,12 @@ public class JsonFactory
     }
 
     /**
+     * Optional method for constructing parser for reading contents from specified {@link DataInput}
+     * instance.
+     *<p>
+     * If this factory does not support {@link DataInput} as source,
+     * will throw {@link UnsupportedOperationException}
+     *
      * @since 2.8
      */
     public JsonParser createParser(DataInput in) throws IOException {
@@ -921,6 +937,34 @@ public class JsonFactory
         return _createParser(_decorate(in, ctxt), ctxt);
     }
 
+    /*
+    /**********************************************************
+    /* Parser factories, non-blocking (async) sources
+    /**********************************************************
+     */
+
+    /**
+     * Optional method for constructing parser for non-blocking parsing
+     * via {@link com.fasterxml.jackson.core.async.ByteArrayFeeder}
+     * interface (accessed using {@link JsonParser#getNonBlockingInputFeeder()}
+     * from constructed instance).
+     *<p>
+     * If this factory does not support non-blocking parsing (either at all,
+     * or from byte array),
+     * will throw {@link UnsupportedOperationException}
+     *
+     * @since 2.9
+     */
+    public JsonParser createNonBlockingByteArrayParser() throws IOException
+    {
+        // 17-May-2017, tatu: Need to take care not to accidentally create JSON parser
+        //   for non-JSON input:
+        _requireJSONFactory("Non-blocking source not (yet?) support for this format (%s)");
+        IOContext ctxt = _createNonBlockingContext(null);
+        ByteQuadsCanonicalizer can = _byteSymbolCanonicalizer.makeChild(_factoryFeatures);
+        return new NonBlockingJsonParser(ctxt, _parserFeatures, can);
+    }
+
     /*
     /**********************************************************
     /* Parser factories (old ones, pre-2.2)
@@ -1320,17 +1364,15 @@ public class JsonFactory
     }
 
     /**
+     * Optional factory method, expected to be overridden
+     *
      * @since 2.8
      */
     protected JsonParser _createParser(DataInput input, IOContext ctxt) throws IOException
     {
         // 13-May-2016, tatu: Need to take care not to accidentally create JSON parser for
-        //   non-JSON input. So, bit unclean but...
-        String format = getFormatName();
-        if (format != FORMAT_NAME_JSON) { // NOTE: only ensure override; full equality NOT needed
-            throw new UnsupportedOperationException(String.format(
-                    "InputData source not (yet?) support for this format (%s)", format));
-        }
+        //   non-JSON input.
+        _requireJSONFactory("InputData source not (yet?) support for this format (%s)");
         // Also: while we can't do full bootstrapping (due to read-ahead limitations), should
         // at least handle possible UTF-8 BOM
         int firstByte = ByteSourceJsonBootstrapper.skipUTF8BOM(input);
@@ -1492,15 +1534,8 @@ public class JsonFactory
          *   scheme, for cases where it is considered harmful (possibly
          *   on Android, for example)
          */
-        if (isEnabled(Feature.USE_THREAD_LOCAL_FOR_BUFFER_RECYCLING)) {
-            SoftReference<BufferRecycler> ref = _recyclerRef.get();
-            BufferRecycler br = (ref == null) ? null : ref.get();
-    
-            if (br == null) {
-                br = new BufferRecycler();
-                _recyclerRef.set(new SoftReference<BufferRecycler>(br));
-            }
-            return br;
+        if (Feature.USE_THREAD_LOCAL_FOR_BUFFER_RECYCLING.enabledIn(_factoryFeatures)) {
+            return BufferRecyclers.getBufferRecycler();
         }
         return new BufferRecycler();
     }
@@ -1513,6 +1548,19 @@ public class JsonFactory
         return new IOContext(_getBufferRecycler(), srcRef, resourceManaged);
     }
 
+    /**
+     * Overridable factory method that actually instantiates desired
+     * context object for async (non-blocking) parsing
+     *
+     * @since 2.9.7
+     */
+    protected IOContext _createNonBlockingContext(Object srcRef) {
+        // [jackson-core#476]: disable buffer recycling for 2.9 to avoid concurrency issues;
+        // easiest done by just constructing private "recycler":
+        BufferRecycler recycler = new BufferRecycler();
+        return new IOContext(recycler, srcRef, false);
+    }
+
     /**
      * @since 2.8
      */
@@ -1546,4 +1594,34 @@ public class JsonFactory
         }
         return url.openStream();
     }
+
+    /*
+    /**********************************************************
+    /* Internal helper methods
+    /**********************************************************
+     */
+
+    /**
+     * Helper method called to work around the problem of this class both defining
+     * general API for constructing parsers+generators AND implementing the API
+     * for JSON handling. Problem here is that when adding new functionality
+     * via factory methods, it is not possible to leave these methods abstract
+     * (because we are implementing them for JSON); but there is risk that
+     * sub-classes do not override them all (plus older version can not implement).
+     * So a work-around is to add a check to ensure that factory is still one
+     * used for JSON; and if not, make base implementation of a factory method fail.
+     *
+     * @since 2.9
+     */
+    private final void _requireJSONFactory(String msg) {
+        if (!_isJSONFactory()) {
+            throw new UnsupportedOperationException(String.format(msg, getFormatName()));
+        }
+    }
+
+    private final boolean _isJSONFactory() {
+        // NOTE: since we only really care about whether this is standard JSON-backed factory,
+        // or its sub-class / delegated to one, no need to check for equality, identity is enough
+        return getFormatName() == FORMAT_NAME_JSON;
+    }
 }
