@@ -422,6 +422,7 @@ public abstract class BaseTest
     {
         StringWriter sw = new StringWriter(100);
         JsonGenerator g = f.createGenerator(sw);
+        g.disable(JsonGenerator.Feature.AUTO_CLOSE_JSON_CONTENT);
         try {
             while (p.nextToken() != null) {
                 g.copyCurrentEvent(p);
@@ -504,6 +505,14 @@ public abstract class BaseTest
     /**********************************************************
      */
 
+    protected static String quote(String str) {
+        return '"'+str+'"';
+    }
+
+    protected static String aposToQuotes(String json) {
+        return json.replace("'", "\"");
+    }
+
     protected byte[] encodeInUTF32BE(String input)
     {
         int len = input.length();
@@ -518,12 +527,13 @@ public abstract class BaseTest
         return result;
     }
 
-    protected String quote(String str) {
-        return '"'+str+'"';
-    }
-
-    protected String aposToQuotes(String json) {
-        return json.replace("'", "\"");
+    // @since 2.9.7
+    protected static byte[] utf8Bytes(String str) {
+        try {
+            return str.getBytes("UTF-8");
+        } catch (IOException e) {
+            throw new RuntimeException(e);
+        }
     }
 
     protected void fieldNameFor(StringBuilder sb, int index)
@@ -547,6 +557,16 @@ public abstract class BaseTest
         }
     }
 
+    // @since 2.9.7
+    protected JsonFactory sharedStreamFactory() {
+        return JSON_FACTORY;
+    }
+
+    // @since 2.9.7
+    protected JsonFactory newStreamFactory() {
+        return new JsonFactory();
+    }
+    
     protected String fieldNameFor(int index)
     {
         StringBuilder sb = new StringBuilder(16);
