@@ -108,7 +108,7 @@ public class NumberParsingTest
         _testSimpleLong(MODE_READER);
         _testSimpleLong(MODE_DATA_INPUT);
     }
-    
+
     private void _testSimpleLong(int mode) throws Exception
     {
         long EXP_L = 12345678907L;
@@ -309,39 +309,6 @@ public class NumberParsingTest
         p.close();
     }
 
-    public void testLongOverflow() throws Exception
-    {
-        BigInteger below = BigInteger.valueOf(Long.MIN_VALUE);
-        below = below.subtract(BigInteger.ONE);
-        BigInteger above = BigInteger.valueOf(Long.MAX_VALUE);
-        above = above.add(BigInteger.ONE);
-
-        String DOC_BELOW = below.toString() + " ";
-        String DOC_ABOVE = below.toString() + " ";
-
-        for (int mode : ALL_MODES) {
-            JsonParser p = createParser(mode, DOC_BELOW);
-            p.nextToken();
-            try {
-                long x = p.getLongValue();
-                fail("Expected an exception for underflow (input "+p.getText()+"): instead, got long value: "+x);
-            } catch (JsonParseException e) {
-                verifyException(e, "out of range of long");
-            }
-            p.close();
-
-            p = createParser(mode, DOC_ABOVE);
-            p.nextToken();
-            try {
-                long x = p.getLongValue();
-                fail("Expected an exception for underflow (input "+p.getText()+"): instead, got long value: "+x);
-            } catch (JsonParseException e) {
-                verifyException(e, "out of range of long");
-            }
-            p.close();
-            
-        }
-    }
     
     /**
      * Method that tries to test that number parsing works in cases where
@@ -461,8 +428,16 @@ public class NumberParsingTest
      */
     public void testParsingOfLongerSequencesWithNonNumeric() throws Exception
     {
-        JsonFactory factory = new JsonFactory();
-        factory.enable(JsonParser.Feature.ALLOW_NON_NUMERIC_NUMBERS);
+        JsonFactory f = new JsonFactory();
+        f.enable(JsonParser.Feature.ALLOW_NON_NUMERIC_NUMBERS);
+        _testParsingOfLongerSequencesWithNonNumeric(f, MODE_INPUT_STREAM);
+        _testParsingOfLongerSequencesWithNonNumeric(f, MODE_INPUT_STREAM_THROTTLED);
+        _testParsingOfLongerSequencesWithNonNumeric(f, MODE_READER);
+        _testParsingOfLongerSequencesWithNonNumeric(f, MODE_DATA_INPUT);
+    }
+
+    private void _testParsingOfLongerSequencesWithNonNumeric(JsonFactory f, int mode) throws Exception
+    {
         double[] values = new double[] {
                 0.01, -10.5, 2.1e9, 4.0e-8,
                 Double.NaN, Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY
@@ -479,16 +454,20 @@ public class NumberParsingTest
             sb.append(arrayJson);
             String DOC = sb.toString();
             for (int input = 0; input < 2; ++input) {
-                JsonParser p;
-                if (input == 0) {
-                    p = createParserUsingStream(factory, DOC, "UTF-8");
-                } else {
-                    p = factory.createParser(DOC);
-                }
+                JsonParser p = createParser(f, mode, DOC);
                 assertToken(JsonToken.START_ARRAY, p.nextToken());
                 for (int j = 0; j < VCOUNT; ++j) {
                     assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
-                    assertEquals(values[i], p.getDoubleValue());
+                    double exp = values[i];
+                    double act = p.getDoubleValue();
+                    if (Double.compare(exp, act) != 0) {
+                        fail("Expected at #"+j+" value "+exp+", instead got "+act);
+                    }
+                    if (Double.isNaN(exp) || Double.isInfinite(exp)) {
+                        assertTrue(p.isNaN());
+                    } else {
+                        assertFalse(p.isNaN());
+                    }
                 }
                 assertToken(JsonToken.END_ARRAY, p.nextToken());
                 p.close();
@@ -594,6 +573,19 @@ public class NumberParsingTest
         assertNull(p.nextToken());
     }
 
+    public void testInvalidNumber() throws Exception {
+        for (int mode : ALL_MODES) {
+            JsonParser p = createParser(mode, " -foo ");
+            try {
+                p.nextToken();
+                fail("Should not pass");
+            } catch (JsonParseException e) {
+                verifyException(e, "Unexpected character ('f'");
+            }
+            p.close();
+        }
+    }
+
     /*
     /**********************************************************
     /* Helper methods
